#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['dj_tables', 'dj_tables.templatetags']

package_data = \
{'': ['*'], 'dj_tables': ['templates/*', 'templates/dj_tables/*']}

install_requires = \
['Django>=2.0.0']

setup(name='dj-tables',
      version='1.0.0',
      description='A template only and DRY solution for creating tables.',
      author='Zach Perkitny',
      author_email='zperkitny@gmail.com',
      url='https://github.com/ninjasforhealth/dj-tables',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      python_requires='>=3.6',
     )
